#ifndef OFFBOARD_COMMUNICATION_H_
#define OFFBOARD_COMMUNICATION_H_

#include "../Process/AsyncProcess.h"
#include <SerialPort.h>
#include <vector>
#include <Rhesus.Toolkit.Tasks.h>

#include <Rhesus/Messenger/NetServer.h>
#include <Rhesus/Messenger/NetBuffer.h>
#include <Rhesus/Messenger/NetChannel.h>

/*!
 * @brief Intermediate interface for serial communications between cRIO and offboard computer. Assumes that the stream has been properly escaped by the offboard computer.
 */
class OffboardCommunication
{
public:
	enum Stream
	{
		UNSET = -1,
		HOT_GOAL = 0,
		GAME_PIECE_TRACKING = 1,
		LIDAR = 2,
	};
	enum Flag
	{
		HOT_GOAL_FLAG = 0x00,
		GAME_PIECE_TRACKING_FLAG = 0x01,
		LIDAR_FLAG = 0x02,
		ESCAPE = 0x7D
	};
	enum ReadState
	{
		WAIT_HEADER,
		IN_MSG,
		AFTER_ESC
	};
	
	static OffboardCommunication* Instance();
	static void Initialize();
	static void Finalize();

	~OffboardCommunication();
	
	/*!
	 * @brief Read the current message from the specified stream.
	 * @param stream the stream to read from
	 * @param buffer the buffer to read the message into
	 */
	void Read(Stream stream, std::vector<char> &buffer);
	
	void Update();
	
private:
	OffboardCommunication();
	
	void Start();
	
	inline Stream FlagToStream(Flag flag)
	{
		if (flag == HOT_GOAL_FLAG)
			return HOT_GOAL;
		if (flag == GAME_PIECE_TRACKING_FLAG)
			return GAME_PIECE_TRACKING;
		if (flag == LIDAR_FLAG)
			return LIDAR;
		return UNSET;
	}
	
	static OffboardCommunication* m_instance;
	
	SerialPort* m_serial;
	
	std::vector<UINT8> buffers[3];
	
	Stream currentStream;
	ReadState currentState;
	
	Rhesus::Messenger::NetServer m_server;
	
	Rhesus::Toolkit::Tasks::Mutex m_syncRoot[3];
};

#endif /* OFFBOARD_COMMUNICATION_H_ */
