#include "NetClient.h"
#include "NetConnection.h"
#include "NetBuffer.h"

using namespace Rhesus::Messenger;

NetClient::NetClient()
	: NetPeer("0.0.0.0", 0, NetConnectionType::CLIENT)
{
	
}

void NetClient::Connect(sockaddr_in ep)
{
	if(_connectionRequested)
		return;
	
	_connectionRequestTime = TIME_IN_SECONDS();
	_connectionRequested = true;
	
	_server = new NetConnection(ep, this);
	
	NetBuffer netBuffer;
	
	netBuffer.Write((UINT8)LIBRARY_DATA);
	netBuffer.Write((UINT8)LibraryMessageType::CONNECTION_REQUEST);
	
	SendRaw(&netBuffer, _server);
}

void NetClient::Disconnect()
{
	if(!_connected) return;
	
	NetBuffer netBuffer;
	
	netBuffer.Write((UINT8)LIBRARY_DATA);
	netBuffer.Write((UINT8)LibraryMessageType::DISCONNECT_REQUEST);

	SendRaw(&netBuffer, _server);
	
	_connected = false;
}

void NetClient::Send(NetBuffer* buff, NetChannel::Enum method, int channel)
{
	NetPeer::Send(buff, _server, method, channel);
}

void NetClient::CheckMessages()
{
	NetPeer::CheckMessages();
	
	if(_connectionRequested && !_connected)
	{
		double timeNow = TIME_IN_SECONDS();
		
		if((timeNow - _connectionRequestTime) > 1.00)
		{
			NetBuffer netBuffer;
				
			netBuffer.Write((UINT8)LIBRARY_DATA);
			netBuffer.Write((UINT8)LibraryMessageType::CONNECTION_REQUEST);
			
			SendRaw(&netBuffer, _server);
			
			_connectionRequestTime = timeNow;
		}
	}
}
